/*
 *  Copyright (C) Chaperon. All rights reserved.                               
 *  ------------------------------------------------------------------------- 
 *  This software is published under the terms of the Apache Software License 
 *  version 1.1, a copy of which has been included  with this distribution in 
 *  the LICENSE file.                                                         
 */

package net.sourceforge.chaperon.parser.generator;

import net.sourceforge.chaperon.grammar.token.definition.DefinitionElement;
import net.sourceforge.chaperon.grammar.token.definition.Alternation;
import net.sourceforge.chaperon.grammar.token.definition.BeginOfLine;
import net.sourceforge.chaperon.grammar.token.definition.CharacterClass;
import net.sourceforge.chaperon.grammar.token.definition.CharacterSequence;
import net.sourceforge.chaperon.grammar.token.definition.Concatenation;
import net.sourceforge.chaperon.grammar.token.definition.EndOfFile;
import net.sourceforge.chaperon.grammar.token.definition.EndOfLine;
import net.sourceforge.chaperon.grammar.token.definition.UniversalCharacter;
import net.sourceforge.chaperon.helpers.IntegerList;
import net.sourceforge.chaperon.parser.RegexAutomate;


/**
 * The generator generates a regex automate from a token definition
 *
 * @author Stephan Michels
 * @version CVS $Id: RegexAutomateGenerator.java,v 1.1 2002/05/06 16:22:57 benedikta Exp $
 */
public class RegexAutomateGenerator 
{
  private DefinitionElement _definitionelement = null;
  private RegexAutomate _automate = null;

  public RegexAutomateGenerator(DefinitionElement definitionelement)
  {
    _definitionelement = definitionelement;
  }

  public RegexAutomate getRegexAutomate()
  {
    if (_automate==null)
    {
      RegexAutomate automate = new RegexAutomate();
      traverse(automate, _definitionelement);
      _automate = automate;
    }

    return _automate;
  }

  private void traverse(RegexAutomate automate, DefinitionElement element)
  {
    for (int i = 0; i < element.getMinOccurs(); i++)
    {
      if (element instanceof Alternation)
        evalAlternation(automate, (Alternation) element);
      else if (element instanceof BeginOfLine)
        evalBeginOfLine(automate, (BeginOfLine) element);
      else if (element instanceof CharacterClass)
        evalCharacterClass(automate, (CharacterClass) element);
      else if (element instanceof CharacterSequence)
        evalCharacterSequence(automate, (CharacterSequence) element);
      else if (element instanceof Concatenation)
        evalConcatenation(automate, (Concatenation) element);
      else if (element instanceof EndOfFile)
        evalEndOfFile(automate, (EndOfFile) element);
      else if (element instanceof EndOfLine)
        evalEndOfLine(automate, (EndOfLine) element);
      else if (element instanceof UniversalCharacter)
        evalUniversalCharacter(automate, (UniversalCharacter) element);
      else
        throw new IllegalArgumentException("Definition element not recognized");
    }

    if (element.getMaxOccurs() < Integer.MAX_VALUE)
    {
      IntegerList labels = new IntegerList();

      for (int i = element.getMinOccurs(); i < element.getMaxOccurs(); i++)
      {
        labels.add(automate.addState(new int[]{ automate.getSize() + 1 }));

        if (element instanceof Alternation)
          evalAlternation(automate, (Alternation) element);
        else if (element instanceof BeginOfLine)
          evalBeginOfLine(automate, (BeginOfLine) element);
        else if (element instanceof CharacterClass)
          evalCharacterClass(automate, (CharacterClass) element);
        else if (element instanceof CharacterSequence)
          evalCharacterSequence(automate, (CharacterSequence) element);
        else if (element instanceof Concatenation)
          evalConcatenation(automate, (Concatenation) element);
        else if (element instanceof EndOfFile)
          evalEndOfFile(automate, (EndOfFile) element);
        else if (element instanceof EndOfLine)
          evalEndOfLine(automate, (EndOfLine) element);
        else if (element instanceof UniversalCharacter)
          evalUniversalCharacter(automate, (UniversalCharacter) element);
        else
          throw new IllegalArgumentException("Definition element not recognized");
      }
      for (int i = element.getMinOccurs(); i < element.getMaxOccurs(); i++)
        automate.addTransition(labels.get(i - element.getMinOccurs()),
        // automate.getSize() + 1);
        automate.getSize()); // experimentell
    }
    else                     // for max = infinity
    {
      int begin = automate.addState(new int[]{ automate.getSize() + 1 });

      if (element instanceof Alternation)
        evalAlternation(automate, (Alternation) element);
      else if (element instanceof BeginOfLine)
        evalBeginOfLine(automate, (BeginOfLine) element);
      else if (element instanceof CharacterClass)
        evalCharacterClass(automate, (CharacterClass) element);
      else if (element instanceof CharacterSequence)
        evalCharacterSequence(automate, (CharacterSequence) element);
      else if (element instanceof Concatenation)
        evalConcatenation(automate, (Concatenation) element);
      else if (element instanceof EndOfFile)
          evalEndOfFile(automate, (EndOfFile) element);
      else if (element instanceof EndOfLine)
        evalEndOfLine(automate, (EndOfLine) element);
      else if (element instanceof UniversalCharacter)
        evalUniversalCharacter(automate, (UniversalCharacter) element);
      else
        throw new IllegalArgumentException("Definition element not recognized");

      automate.addTransition(begin, automate.getSize() + 1);
      automate.addState(new int[]{ automate.getSize() + 1, begin + 1 });
    }
    automate.setAcceptState(automate.getSize());
  }

  private void evalAlternation(RegexAutomate automate, Alternation element)
  {
    if (element.getDefinitionElementCount() == 1)
      traverse(automate, element.getDefinitionElement(0));
    else
    {
      int lastbegin = -1;
      int lastend = -1;

      for (int i = 0; i < element.getDefinitionElementCount(); i++)
      {
        // For a jump to an alternative, but at the end
        if (i < (element.getDefinitionElementCount() - 1))
          lastbegin = automate.addState(new int[]{ automate.getSize() + 1 });

        traverse(automate, element.getDefinitionElement(i));

        // sets the last jumplabel
        if (lastend != -1)
          automate.addTransition(lastend, automate.getSize());

        if (i < (element.getDefinitionElementCount() - 1))
        {
          // sets the last begin jumplabel
          automate.addTransition(lastbegin, automate.getSize() + 1);

          // sets the jumplabel to the end
          lastend = automate.addState();
        }
      }
    }
  }

  private void evalBeginOfLine(RegexAutomate automate, BeginOfLine element)
  {
    automate.addState(RegexAutomate.TYPE_BOL, new int[]{ automate.getSize() + 1 });
  }

  private void evalCharacterClass(RegexAutomate automate, CharacterClass element)
  {
    StringBuffer buffer = new StringBuffer();

    for (int i = 0; i < element.getCharacterClassElementCount(); i++)
      buffer.append(element.getCharacterClassElement(i).getCharacters());
    String characters = buffer.toString();

    automate.addState(characters, element.isNegation()?RegexAutomate.TYPE_NEGMATCH:RegexAutomate.TYPE_MATCH,
                      new int[]{ automate.getSize() + 1 });
  }
  
  private void evalCharacterSequence(RegexAutomate automate, CharacterSequence element)
  {
    for (int i = 0; i < element.getSequence().length(); i++)
      automate.addState(element.getSequence().substring(i, i + 1), RegexAutomate.TYPE_MATCH,
                        new int[]{ automate.getSize() + 1 });
  }

  private void evalConcatenation(RegexAutomate automate, Concatenation element)
  {
    for (int i = 0; i < element.getDefinitionElementCount(); i++)
      traverse(automate, element.getDefinitionElement(i));
  }

  private void evalEndOfFile(RegexAutomate automate, EndOfFile element)
  {
    automate.addState(RegexAutomate.TYPE_EOF, new int[]{ automate.getSize() + 1 });
  }

  private void evalEndOfLine(RegexAutomate automate, EndOfLine element)
  {
    automate.addState(RegexAutomate.TYPE_EOL, new int[]{ automate.getSize() + 1 });
  }

  private void evalUniversalCharacter(RegexAutomate automate, UniversalCharacter element)
  {
    automate.addState(RegexAutomate.TYPE_MATCHANY, new int[]{ automate.getSize() + 1 });
  }
}
